//
//  BtViewController.m
//  GeliBusinessPlatform
//
//  Created by junye lu on 2021/2/26.
//  Copyright © 2021 junye lu. All rights reserved.
//

#import "BtViewController.h"
#import "HLBLEManager.h"
#import "SVProgressHUD.h"
#import "HLPrinter.h"
#import "WKWebView+UIImage.h"
#import "Bt_Cell.h"


@interface BtViewController ()<UITableViewDelegate,UITableViewDataSource>
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (strong, nonatomic)NSMutableArray  *devices;  /**< 蓝牙设备个数 */
@property (strong, nonatomic)NSMutableArray  *selectArray;//选择蓝牙
@property(nonatomic,strong)NSMutableDictionary *dicts;

@end

@implementation BtViewController

-(NSMutableArray *)devices {
    if (!_devices) {
        _devices = [[NSMutableArray alloc]init];
    }
    return _devices;
}

-(NSMutableDictionary *)dicts {
    if (!_dicts) {
        _dicts = [[NSMutableDictionary alloc]init];
    }
    return _dicts;
}

- (IBAction)backAction:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}


-(void)dealloc{
    [Manager close];
}
- (void)viewDidLoad {
    [super viewDidLoad];

    // Do any additional setup after loading the view from its nib.
    if (!_selectArray) {
        _selectArray = [NSMutableArray array];
    }
    _tableView.rowHeight = 44;
    
    [_tableView registerNib:[UINib nibWithNibName:@"Bt_Cell" bundle:NULL] forCellReuseIdentifier:@"Bt_Cell"];
    
}

-(void)viewDidAppear:(BOOL)animated {
    if (Manager.bleConnecter == nil) {
        [Manager didUpdateState:^(NSInteger state) {
            switch (state) {
                case CBCentralManagerStateUnsupported:
                    NSLog(@"The platform/hardware doesn't support Bluetooth Low Energy.");
                    break;
                case CBCentralManagerStateUnauthorized:
                    NSLog(@"The app is not authorized to use Bluetooth Low Energy.");
                    break;
                case CBCentralManagerStatePoweredOff:
                    NSLog(@"Bluetooth is currently powered off.");
                    break;
                case CBCentralManagerStatePoweredOn:
                    [self startScane];
                    NSLog(@"Bluetooth power on");
                    break;
                case CBCentralManagerStateUnknown:
                default:
                    break;
            }
        }];
    } else {
        [self startScane];
    }
}

-(void)startScane {
    [Manager scanForPeripheralsWithServices:nil options:nil discover:^(CBPeripheral * _Nullable peripheral, NSDictionary<NSString *,id> * _Nullable advertisementData, NSNumber * _Nullable RSSI) {
        if (peripheral.name != nil) {
            NSLog(@"name -> %@",peripheral.name);
            NSUInteger oldCounts = [self.dicts count];
            [self.dicts setObject:peripheral forKey:peripheral.identifier.UUIDString];
            if (oldCounts < [self.dicts count]) {
                [_tableView reloadData];
            }
        }
    }];
}
-(void)viewDidDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [Manager stopScan];
 
}


#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section == 0) {
        return _selectArray.count;
    }
    return [[self.dicts allKeys]count];;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    [Manager closePeripheral: _selectArray.firstObject];
    [self.selectArray removeAllObjects];
    if (indexPath.section == 1) {
        CBPeripheral *peripheral = [self.dicts objectForKey:[self.dicts allKeys][indexPath.row]];
        [self connectDevice:peripheral];
    }else{
        [SVProgressHUD showErrorWithStatus:@"断开连接"];
    }
    [_tableView reloadData];
}

-(void)connectDevice:(CBPeripheral *)peripheral {
    
    [Manager connectPeripheral:peripheral options:nil timeout:2 connectBlack:^(ConnectState state) {
        if (state == 3) {
            if ([_selectArray count] == 0) {
                [self.selectArray addObject:peripheral];
                
                [_tableView reloadData];
            }
            [SVProgressHUD showSuccessWithStatus:@"连接成功"];
        }
        if (state == 5){
            [SVProgressHUD showErrorWithStatus:@"连接失败"];
            
        }
    }];
}




- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    Bt_Cell *cell = [tableView dequeueReusableCellWithIdentifier:@"Bt_Cell"];
    CBPeripheral *peripheral = [self.dicts objectForKey:[self.dicts allKeys][indexPath.row]];
    cell.title_lbl.text = peripheral.name;
    cell.statusImg.hidden = YES;
    if (indexPath.section == 0) {
        CBPeripheral *p = _selectArray[0];
        cell.statusImg.hidden = NO;
        cell.title_lbl.text = p.name;
    }
    
    return cell;
}
-(CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 46;
}
-(CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
    return 46;
}
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *view = [[UIView alloc]initWithFrame:CGRectMake(0, 0, self.view.bounds.size.width, 46)];
    view.backgroundColor = [UIColor colorNamed:@"app底色"];
    UILabel *lbl = [[UILabel alloc]initWithFrame:CGRectMake(15, 0, self.view.bounds.size.width, 46)];
    lbl.font = [UIFont boldSystemFontOfSize:15];
    lbl.textColor = [UIColor colorNamed:@"灰色字体颜色"];
    [view addSubview:lbl];
    switch (section) {
        case 0:
        {
            lbl.text = @"已连接设备";
        }
            break;
            
        default:
        {
            lbl.text = @"未连接设备";
        }
            break;
    }
    return view;
}
- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section{
    UIView *view = [[UIView alloc]initWithFrame:CGRectMake(0, 0, self.view.bounds.size.width, 46)];
    view.backgroundColor = [UIColor colorNamed:@"app底色"];
    UILabel *lbl = [[UILabel alloc]initWithFrame:CGRectMake(15, 0, self.view.bounds.size.width, 46)];
    lbl.font = [UIFont boldSystemFontOfSize:11];
    lbl.textColor = [UIColor colorNamed:@"灰色字体颜色"];
    [view addSubview:lbl];
    switch (section) {
        case 0:
        {
            lbl.text = @"选择已经连接的设备作为默认打印设备";
        }
            break;
        default:
        {
            lbl.text = @"选择设备进行连接";
        }
            break;
    }
    return view;
}


@end
